@@ -1,11 +1,13 @@
 #include <config.h>
-#include <libnautilus/libnautilus.h>
+
 #include "hyperbola-filefmt.h"
 #include <gtk/gtk.h>
+#include <gtk/gtkctree.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
-#include <libnautilus-extensions/nautilus-ctree.h>
-#include <libnautilus-extensions/nautilus-gdk-font-extensions.h>
+#include <libgnomeui/gnome-uidefs.h>
+#include <libnautilus-private/nautilus-theme.h>
+#include <libnautilus/libnautilus.h>
 
 #include "hyperbola-nav.h"
 
@@ -27,7 +29,20 @@ typedef struct {
 
 	gint notify_count;
 
-	GtkStyle *italic_style;
+	GdkPixmap *folder_open_pixmap;
+	GdkBitmap *folder_open_mask;
+	GdkPixmap *folder_closed_pixmap;
+	GdkBitmap *folder_closed_mask;
+
+	GdkPixmap *book_open_pixmap;
+	GdkBitmap *book_open_mask;
+	GdkPixmap *book_closed_pixmap;
+	GdkBitmap *book_closed_mask;
+
+	GdkPixmap *section_open_pixmap;
+	GdkBitmap *section_open_mask;
+	GdkPixmap *section_closed_pixmap;
+	GdkBitmap *section_closed_mask;
 } HyperbolaNavigationTree;
 
 static void hyperbola_navigation_tree_destroy (GtkCTree * ctree,
@@ -51,41 +66,84 @@ typedef struct {
 } PopulateInfo;
 
 static void
-ensure_italic_style (HyperbolaNavigationTree *view)
+ensure_pixmap_and_mask (GdkPixmap **pixmap, GdkBitmap **mask, const char *name)
 {
-	if (view->italic_style == NULL) {
-		/* Set italic style for page names */
-		GdkFont *font;
-		GtkStyle *style = gtk_style_copy
-			(gtk_widget_get_style (view->ctree));
+	char *fullname;
+	GdkPixbuf *pixbuf;
 
-		font = style->font;
-		style->font = nautilus_gdk_font_get_italic (font);
-		gdk_font_unref (font);
-
-		view->italic_style = style;
+	if (*pixmap != NULL) {
+		return;
 	}
+
+	fullname = nautilus_theme_get_image_path (name);
+	if (fullname == NULL) {
+		return;
+	}
+
+	pixbuf = gdk_pixbuf_new_from_file (fullname);
+	if (pixbuf == NULL) {
+		return;
+	}
+
+	gdk_pixbuf_render_pixmap_and_mask (pixbuf,
+					   pixmap,
+					   mask,
+					   127 /* alpha_threshold */);
+
+	g_free (fullname);
 }
 
 static void
-set_node_style (HyperbolaNavigationTree *view,
-		GtkCTree *ctree, GtkCTreeNode *node)
+ensure_icons (HyperbolaNavigationTree *view)
 {
-	HyperbolaTreeNode *tnode;
+	ensure_pixmap_and_mask (&view->folder_open_pixmap,
+				&view->folder_open_mask,
+				"hyperbola-folder-open.png");
+	ensure_pixmap_and_mask (&view->folder_closed_pixmap,
+				&view->folder_closed_mask,
+				"hyperbola-folder-closed.png");
 
-	g_assert (GTK_IS_CTREE (ctree));
+	ensure_pixmap_and_mask (&view->book_open_pixmap,
+				&view->book_open_mask,
+				"hyperbola-book-open.png");
+	ensure_pixmap_and_mask (&view->book_closed_pixmap,
+				&view->book_closed_mask,
+				"hyperbola-book-closed.png");
 
-	tnode = gtk_ctree_node_get_row_data (ctree, node);
+	ensure_pixmap_and_mask (&view->section_open_pixmap,
+				&view->section_open_mask,
+				"hyperbola-section-open.png");
+	ensure_pixmap_and_mask (&view->section_closed_pixmap,
+				&view->section_closed_mask,
+				"hyperbola-section-closed.png");
+}
 
-	if (tnode != NULL
-	    && (tnode->type == HYP_TREE_NODE_PAGE
-		|| tnode->type == HYP_TREE_NODE_BOOK)) {
-		ensure_italic_style (view);
+static void
+get_node_icons (HyperbolaNavigationTree *view,
+		HyperbolaTreeNode *node,
+		GdkPixmap **pixmap_closed,
+		GdkBitmap **mask_closed,
+		GdkPixmap **pixmap_opened,
+		GdkBitmap **mask_opened)
+{
+	ensure_icons (view);
 
-		gtk_ctree_node_set_row_style (ctree, node, view->italic_style);
-	} else {
-		/* no special style */
-		gtk_ctree_node_set_row_style (ctree, node, NULL);
+	if (node->type == HYP_TREE_NODE_FOLDER) {
+		*pixmap_opened = view->folder_open_pixmap;
+		*mask_opened = view->folder_open_mask;
+		*pixmap_closed = view->folder_closed_pixmap;
+		*mask_closed = view->folder_closed_mask;
+	} else if (node->type == HYP_TREE_NODE_BOOK) {
+		*pixmap_opened = view->book_open_pixmap;
+		*mask_opened = view->book_open_mask;
+		*pixmap_closed = view->book_closed_pixmap;
+		*mask_closed = view->book_closed_mask;
+	} else if (node->type == HYP_TREE_NODE_SECTION
+		   || node->type == HYP_TREE_NODE_PAGE) {
+		*pixmap_opened = view->section_open_pixmap;
+		*mask_opened = view->section_open_mask;
+		*pixmap_closed = view->section_closed_pixmap;
+		*mask_closed = view->section_closed_mask;
 	}
 }
 
@@ -96,6 +154,10 @@ ctree_populate_subnode (gpointer key, gpointer value, gpointer user_data)
 	PopulateInfo *pi = (PopulateInfo *) user_data, subpi;
 	gboolean term;
 	char *title;
+	GdkPixmap *pixmap_closed = NULL;
+	GdkBitmap *mask_closed = NULL;
+	GdkPixmap *pixmap_opened = NULL;
+	GdkBitmap *mask_opened = NULL;
 
 #ifdef ENABLE_SCROLLKEEPER_SUPPORT
 	/* Get rid of leading numbers used to make sure TOCs are displayed properly */
@@ -114,16 +176,19 @@ ctree_populate_subnode (gpointer key, gpointer value, gpointer user_data)
 #endif
 		title = node->title;
 
+	get_node_icons (pi->view, node,
+			&pixmap_closed, &mask_closed,
+			&pixmap_opened, &mask_opened);
+
 	term = (node->type == HYP_TREE_NODE_PAGE) || !node->children;
 	pi->sibling =
 		gtk_ctree_insert_node (GTK_CTREE (pi->ctree),
-				       pi->parent, NULL, &title, 5, NULL,
-				       NULL, NULL, NULL, term, FALSE);
+				       pi->parent, NULL, &title, 5, 
+				       pixmap_closed, mask_closed,
+				       pixmap_opened, mask_opened,
+				       term, FALSE);
 	node->user_data = pi->sibling;
 
-	set_node_style (pi->view, GTK_CTREE (pi->ctree), pi->sibling);
-	
-
 #ifdef ENABLE_SCROLLKEEPER_SUPPORT
 	if (title != node->title)
 		g_free (title);	/* We used the copy from the split */
@@ -133,6 +198,7 @@ ctree_populate_subnode (gpointer key, gpointer value, gpointer user_data)
 				     node);
 
 	if (node->children) {
+		subpi.view = pi->view;
 		subpi.ctree = pi->ctree;
 		subpi.sibling = NULL;
 		subpi.parent = pi->sibling;
@@ -167,34 +233,6 @@ ctree_populate (HyperbolaNavigationTree * view)
 			 G_IN_ORDER, &subpi);
 }
 
-static void
-reset_style_for_node (GtkCTree *ctree, GtkCTreeNode *node, gpointer data)
-{
-	HyperbolaNavigationTree *view = data;
-
-	set_node_style (view, ctree, node);
-}
-
-static void
-reset_styles (GtkWidget *widget, GtkStyle *old_style, gpointer data)
-{
-	HyperbolaNavigationTree *view = data;
-
-	g_assert (GTK_IS_CTREE (widget));
-
-	if (view->italic_style != NULL) {
-		gtk_style_unref (view->italic_style);
-		view->italic_style = NULL;
-	}
-
-	gtk_clist_freeze (GTK_CLIST (widget));
-	gtk_ctree_post_recursive (GTK_CTREE (widget),
-				  NULL,
-				  reset_style_for_node,
-				  view);
-	gtk_clist_thaw (GTK_CLIST (widget));
-}
-
 BonoboObject *
 hyperbola_navigation_tree_new (void)
 {
@@ -222,8 +260,6 @@ hyperbola_navigation_tree_new (void)
 			    hyperbola_navigation_tree_select_row, view);
 	gtk_signal_connect (GTK_OBJECT (view->top_ctree), "destroy",
 			    hyperbola_navigation_tree_destroy, view);
-	gtk_signal_connect (GTK_OBJECT (view->top_ctree), "style_set",
-			    reset_styles, view);
 #endif
 
 	view->ctree = gtk_ctree_new (1, 0);
@@ -239,11 +275,7 @@ hyperbola_navigation_tree_new (void)
 			    hyperbola_navigation_tree_select_row, view);
 	gtk_signal_connect (GTK_OBJECT (view->ctree), "destroy",
 			    hyperbola_navigation_tree_destroy, view);
-	gtk_signal_connect (GTK_OBJECT (view->ctree), "style_set",
-			    reset_styles, view);
 
-	view->italic_style = NULL;
-			    
 #ifdef ENABLE_SCROLLKEEPER_SUPPORT
 	view->selected_ctree = NULL;
 	view->selected_node = NULL;
@@ -276,8 +308,10 @@ hyperbola_navigation_tree_new (void)
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (wtmp), box);
 	
 	if (!top_tree_empty) {
-        	label = gtk_label_new (_("Introductory Documents:"));
+        	label = gtk_label_new (_("Introductory Documents"));
 		gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+		gtk_misc_set_padding (GTK_MISC (label), GNOME_PAD_SMALL, 0);
         	gtk_widget_show (label);
         	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
 			
@@ -290,8 +324,10 @@ hyperbola_navigation_tree_new (void)
 	
 	gtk_box_pack_start ( GTK_BOX(box), view->top_ctree, FALSE, FALSE, 0);
 	
-	label = gtk_label_new (_("Documents by Subject:"));
+	label = gtk_label_new (_("Documents by Subject"));
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_misc_set_padding (GTK_MISC (label), GNOME_PAD_SMALL, 0);
         gtk_widget_show (label);
 	gtk_box_pack_start ( GTK_BOX(box), label, FALSE, FALSE, 0);
 #endif 
@@ -304,10 +340,14 @@ hyperbola_navigation_tree_new (void)
 #endif
 
 #ifdef ENABLE_SCROLLKEEPER_SUPPORT
-	gtk_clist_columns_autosize (GTK_CLIST (view->top_ctree));
+	gtk_clist_set_column_auto_resize (GTK_CLIST (view->top_ctree),
+					  0 /* column */,
+					  TRUE /* auto_resize */);
 	gtk_clist_thaw (GTK_CLIST (view->top_ctree));
 #endif
-	gtk_clist_columns_autosize (GTK_CLIST (view->ctree));
+	gtk_clist_set_column_auto_resize (GTK_CLIST (view->ctree),
+					  0 /* column */,
+					  TRUE /* auto_resize */);
 	gtk_clist_thaw (GTK_CLIST (view->ctree));
 		
 	view->view_frame = nautilus_view_new (wtmp);
@@ -454,14 +494,42 @@ hyperbola_navigation_tree_select_row (GtkCTree * ctree, GtkCTreeNode * node,
 	view->notify_count--;
 }
 
+static void
+pixmap_unref_and_null (GdkPixmap **pixmap)
+{
+	if (*pixmap != NULL) {
+		gdk_pixmap_unref (*pixmap);
+		*pixmap = NULL;
+	}
+}
+
+static void
+bitmap_unref_and_null (GdkPixmap **bitmap)
+{
+	if (*bitmap != NULL) {
+		gdk_bitmap_unref (*bitmap);
+		*bitmap = NULL;
+	}
+}
+
 static void
 hyperbola_navigation_tree_destroy (GtkCTree * ctree,
 				   HyperbolaNavigationTree * view)
 {
 	set_pending_location (view, NULL);
 
-	if (view->italic_style != NULL) {
-		gtk_style_unref (view->italic_style);
-		view->italic_style = NULL;
-	}
+	pixmap_unref_and_null (&view->folder_open_pixmap);
+	bitmap_unref_and_null (&view->folder_open_mask);
+	pixmap_unref_and_null (&view->folder_closed_pixmap);
+	bitmap_unref_and_null (&view->folder_closed_mask);
+
+	pixmap_unref_and_null (&view->book_open_pixmap);
+	bitmap_unref_and_null (&view->book_open_mask);
+	pixmap_unref_and_null (&view->book_closed_pixmap);
+	bitmap_unref_and_null (&view->book_closed_mask);
+
+	pixmap_unref_and_null (&view->section_open_pixmap);
+	bitmap_unref_and_null (&view->section_open_mask);
+	pixmap_unref_and_null (&view->section_closed_pixmap);
+	bitmap_unref_and_null (&view->section_closed_mask);
 }
