@@ -19,7 +19,7 @@
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
   
-   Author: Darin Adler <darin@eazel.com>
+   Author: Darin Adler <darin@bentspoon.com>
 */
 
 #include <config.h>
@@ -128,6 +128,8 @@ nautilus_directory_initialize (gpointer object, gpointer klass)
 
 	directory->details = g_new0 (NautilusDirectoryDetails, 1);
 	directory->details->file_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	directory->details->high_priority_queue = nautilus_file_queue_new ();
+	directory->details->low_priority_queue = nautilus_file_queue_new ();
 	directory->details->idle_queue = nautilus_idle_queue_new ();
 }
 
@@ -195,6 +197,8 @@ nautilus_directory_destroy (GtkObject *object)
 	}
 	g_assert (directory->details->file_list == NULL);
 	g_hash_table_destroy (directory->details->file_hash);
+	nautilus_file_queue_destroy (directory->details->high_priority_queue);
+	nautilus_file_queue_destroy (directory->details->low_priority_queue);
 	nautilus_idle_queue_destroy (directory->details->idle_queue);
 	g_assert (directory->details->directory_load_in_progress == NULL);
 	g_assert (directory->details->count_in_progress == NULL);
@@ -306,10 +310,11 @@ nautilus_directory_make_uri_canonical (const char *uri)
 	/* To NautilusDirectory, a uri with or without a trailing
 	 * / is equivalent. This is necessary to prevent separate
 	 * NautilusDirectories for the same location from being
-	 * created. (See bugzilla.eazel.com 3322 for an example.)
+	 * created. (See bugzilla.gnome.org 43322 for an example.)
 	 */
 	canonical = eel_str_strip_trailing_chr (canonical_maybe_trailing_slash, '/');
-	if (strcmp (canonical, canonical_maybe_trailing_slash) != 0) {
+	if (strcmp (canonical, canonical_maybe_trailing_slash) != 0 &&
+	    strcmp (canonical, "favorites:") != 0) {
 		/* If some trailing '/' were stripped, there's the possibility,
 		 * that we stripped away all the '/' from a uri that has only
 		 * '/' characters. If you change this code, check to make sure
@@ -561,6 +566,7 @@ nautilus_directory_add_file (NautilusDirectory *directory, NautilusFile *file)
 	/* Ref if we are monitoring. */
 	if (nautilus_directory_is_file_list_monitored (directory)) {
 		nautilus_file_ref (file);
+		nautilus_directory_add_file_to_work_queue (directory, file);
 	}
 }
 
@@ -583,6 +589,8 @@ nautilus_directory_remove_file (NautilusDirectory *directory, NautilusFile *file
 		(directory->details->file_list, node);
 	g_list_free_1 (node);
 
+	nautilus_directory_remove_file_from_work_queue (directory, file);
+
 	if (!file->details->unconfirmed) {
 		directory->details->confirmed_file_count--;
 	}
@@ -863,7 +871,9 @@ nautilus_directory_notify_files_added (GList *uris)
 	NautilusDirectory *directory;
 	GHashTable *parent_directories;
 	const char *uri;
+	const char *directory_uri;
 	GnomeVFSURI *vfs_uri;
+	NautilusFile *file;
 
 	/* Make a list of added files in each directory. */
 	added_lists = g_hash_table_new (NULL, NULL);
@@ -877,6 +887,19 @@ nautilus_directory_notify_files_added (GList *uris)
 		/* See if the directory is already known. */
 		directory = get_parent_directory_if_exists (uri);
 		if (directory == NULL) {
+			/* In case the directory is not being
+			 * monitored, but the corresponding file is,
+			 * we must invalidate it's item count.
+			 */
+			
+			directory_uri = uri_get_directory_part (uri);
+			file = nautilus_file_get_existing (directory_uri);
+
+			if (file != NULL) {
+				nautilus_file_invalidate_count_and_mime_list (file);
+				nautilus_file_unref (file);
+			}
+
 			continue;
 		}
 
@@ -1444,7 +1467,9 @@ nautilus_directory_file_monitor_add (NautilusDirectory *directory,
 				     gconstpointer client,
 				     gboolean monitor_hidden_files,
 				     gboolean monitor_backup_files,
-				     GList *file_attributes)
+				     GList *file_attributes,
+				     NautilusDirectoryCallback callback,
+				     gpointer callback_data)
 {
 	g_return_if_fail (NAUTILUS_IS_DIRECTORY (directory));
 	g_return_if_fail (client != NULL);
@@ -1454,7 +1479,8 @@ nautilus_directory_file_monitor_add (NautilusDirectory *directory,
 		 file_monitor_add, (directory, client,
 				    monitor_hidden_files,
 				    monitor_backup_files,
-				    file_attributes));
+				    file_attributes,
+				    callback, callback_data));
 }
 
 void
@@ -1489,6 +1515,105 @@ nautilus_directory_is_not_empty (NautilusDirectory *directory)
 		 is_not_empty, (directory));
 }
 
+static gboolean
+is_tentative (gpointer data, gpointer callback_data)
+{
+	NautilusFile *file;
+
+	g_assert (callback_data == NULL);
+
+	file = NAUTILUS_FILE (data);
+	return file->details->info == NULL;
+}
+
+GList *
+nautilus_directory_get_file_list (NautilusDirectory *directory)
+{
+	GList *tentative_files, *non_tentative_files;
+
+	tentative_files = eel_g_list_partition
+		(g_list_copy (directory->details->file_list),
+		 is_tentative, NULL, &non_tentative_files);
+	g_list_free (tentative_files);
+
+	nautilus_file_list_ref (non_tentative_files);
+	return non_tentative_files;
+}
+
+/**
+ * nautilus_directory_list_ref
+ *
+ * Ref all the directories in a list.
+ * @list: GList of directories.
+ **/
+GList *
+nautilus_directory_list_ref (GList *list)
+{
+	g_list_foreach (list, (GFunc) nautilus_directory_ref, NULL);
+	return list;
+}
+
+/**
+ * nautilus_directory_list_unref
+ *
+ * Unref all the directories in a list.
+ * @list: GList of directories.
+ **/
+void
+nautilus_directory_list_unref (GList *list)
+{
+	eel_g_list_safe_for_each (list, (GFunc) nautilus_directory_unref, NULL);
+}
+
+/**
+ * nautilus_directory_list_free
+ *
+ * Free a list of directories after unrefing them.
+ * @list: GList of directories.
+ **/
+void
+nautilus_directory_list_free (GList *list)
+{
+	nautilus_directory_list_unref (list);
+	g_list_free (list);
+}
+
+/**
+ * nautilus_directory_list_copy
+ *
+ * Copy the list of directories, making a new ref of each,
+ * @list: GList of directories.
+ **/
+GList *
+nautilus_directory_list_copy (GList *list)
+{
+	return g_list_copy (nautilus_directory_list_ref (list));
+}
+
+static int
+compare_by_uri (NautilusDirectory *a, NautilusDirectory *b)
+{
+	return strcmp (a->details->uri, b->details->uri);
+}
+
+static int
+compare_by_uri_cover (gconstpointer a, gconstpointer b)
+{
+	return compare_by_uri (NAUTILUS_DIRECTORY (a), NAUTILUS_DIRECTORY (b));
+}
+
+/**
+ * nautilus_directory_list_sort_by_uri
+ * 
+ * Sort the list of directories by directory uri.
+ * @list: GList of directories.
+ **/
+GList *
+nautilus_directory_list_sort_by_uri (GList *list)
+{
+	return g_list_sort (list, compare_by_uri_cover);
+}
+
 #if !defined (NAUTILUS_OMIT_SELF_CHECK)
 
 #include <eel/eel-debug.h>
@@ -1538,7 +1663,7 @@ nautilus_self_check_directory (void)
 
 	nautilus_directory_file_monitor_add
 		(directory, &data_dummy,
-		 TRUE, TRUE, NULL);
+		 TRUE, TRUE, NULL, NULL, NULL);
 
 	got_metadata_flag = FALSE;
 
@@ -1656,7 +1781,7 @@ nautilus_self_check_directory (void)
 	EEL_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("http://le-hackeur.org/"), "http://le-hackeur.org");
 	EEL_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("http://le-hackeur.org/dir"), "http://le-hackeur.org/dir");
 	EEL_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("http://le-hackeur.org/dir/"), "http://le-hackeur.org/dir");
-	/* FIXME bugzilla.eazel.com 5068: the "nested" URI loses some characters here. Maybe that's OK because we escape them in practice? */
+	/* FIXME bugzilla.gnome.org 45068: the "nested" URI loses some characters here. Maybe that's OK because we escape them in practice? */
 	EEL_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("search://[file://]file_name contains stuff"), "search://[file/]file_name contains stuff");
 	EEL_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("eazel-services:/~turtle"), "eazel-services:///~turtle");
 	EEL_CHECK_STRING_RESULT (nautilus_directory_make_uri_canonical ("eazel-services:///~turtle"), "eazel-services:///~turtle");
